@@ -1,7 +1,9 @@
 package Pcore::Ext::Lib::Core::Viewport;
 
 use Pcore -l10n;
+use Pcore::Resources::FA qw[:ALL];
 
+# VIEWPORT CONTROLLER
 sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
     return {
         roles => [],
@@ -40,7 +42,7 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
             },
         },
 
-        init => func [], <<"JS",
+        init => func ['view'], <<"JS",
             var me = this;
 
             // parse API methods
@@ -330,7 +332,7 @@ JS
 
             me.doRecoverPassword(username, function (res) {
                 if (res.isSuccess()) {
-                    Ext.toast($l10n->{'Email with password change instructions was sent to the email address, associated with your user name.'}, 5000);
+                    Ext.toast($l10n->{'Password change instructions was sent to the email address, associated with your account.'}, 5000);
 
                     if (cb) cb(true);
                 }
@@ -371,6 +373,199 @@ JS
     };
 }
 
+# SIGNIN DIALOG
+sub EXT_signin_controller : Extend('Ext.app.ViewController') : Type('controller') {
+    return {
+        submit => func [],
+        <<"JS",
+                var me = this;
+                var view = this.getView();
+                var form = view.down('fieldpanel');
+
+                if (form.validate()) {
+                    Ext.fireEvent('signin',
+                        form.getFields('username').getValue(),
+                        form.getFields('password').getValue(),
+                        this.lookup('remember_me').isChecked(),
+                        function (success) {if (success) view.destroy()}
+                    );
+                }
+JS
+
+        recoverPassword => func [], <<"JS",
+                var me = this;
+                var view = this.getView(),
+                    form = view.down('fieldpanel'),
+                    username_field = form.getFields('username');
+
+                form.clearErrors();
+
+                if (username_field.validate()) {
+                    Ext.fireEvent('mask');
+
+                    Ext.fireEvent('recoverPassword',
+                        username_field.getValue(),
+                        function (success) {
+                            Ext.fireEvent('unmask');
+
+                            form.reset(true);
+                        }
+                    );
+                }
+JS
+    };
+}
+
+sub EXT_signin : Extend('Ext.Dialog') : Type('widget') {
+    return {
+        controller => $type->{signin_controller},
+
+        title        => { text => l10n('SIGN IN') },
+        defaultFocus => 'textfield[name=username]',
+        draggable    => \0,
+        width        => 320,
+
+        keyMap => { ENTER => 'submit', },
+
+        items => [ {
+            xtype => 'fieldpanel',
+
+            items => [
+                {   xtype       => 'textfield',
+                    name        => 'username',
+                    label       => l10n('User name'),
+                    placeholder => l10n('email'),
+                    allowBlank  => \0,
+                    required    => \1,
+                },
+                {   xtype       => 'passwordfield',
+                    name        => 'password',
+                    label       => l10n('Password'),
+                    placeholder => l10n('password'),
+                    allowBlank  => \0,
+                    required    => \1,
+                },
+                {   xtype     => 'checkboxfield',
+                    reference => 'remember_me',
+                    boxLabel  => l10n('Remember me'),
+                    checked   => 1,
+                },
+            ],
+
+            buttons => [
+                {   text    => l10n('Forgot password'),
+                    ui      => 'decline',
+                    handler => 'recoverPassword',
+                },
+                {   text      => l10n('Sign in'),
+                    iconCls   => $FAS_SIGN_IN_ALT,
+                    iconAlign => 'left',
+                    ui        => 'confirm',
+                    handler   => 'submit',
+                },
+            ]
+        } ],
+    };
+}
+
+# CHANGE PASSWORD DIALOG
+sub EXT_change_password_controller : Extend('Ext.app.ViewController') : Type('controller') {
+    return {
+        submit => func [],
+        <<"JS",
+            var me = this;
+            var view = this.getView();
+            var form = view.down('fieldpanel');
+
+            if (form.validate()) {
+                var password = form.getFields('password').getValue(),
+                    password1_field = form.getFields('password1');
+
+                if (password != password1_field.getValue()) {
+                    password1_field.setError($l10n->{'Passwords are not match'});
+
+                    return;
+                }
+
+                Ext.fireEvent('mask');
+
+                Ext.fireEvent('changePassword',
+                    password,
+                    view.getToken(),
+                    function (success) {
+                        Ext.fireEvent('unmask');
+
+                        if (success) me.close()
+                    }
+                );
+            }
+JS
+
+        close => func [], <<"JS",
+            var view = this.getView();
+
+            view.destroy();
+JS
+
+        onDestroy => func [], <<"JS",
+            var view = this.getView(),
+                redirectOnClose = view.getRedirectOnClose();
+
+            if (redirectOnClose != null) this.redirectTo(redirectOnClose);
+JS
+    };
+}
+
+sub EXT_change_password : Extend('Ext.Dialog') : Type('widget') {
+    return {
+        controller => $type->{change_password_controller},
+
+        config => {
+            token           => undef,    # change password token
+            redirectOnClose => undef,    # hash, to redirect to on destroy
+        },
+
+        title        => { text => l10n('PASSWORD CHANGING') },
+        defaultFocus => 'passwordfield[name=password]',
+        draggable    => \0,
+        width        => 320,
+
+        keyMap => { ENTER => 'submit', },
+
+        listeners => { destroy => 'onDestroy' },
+
+        items => [ {
+            xtype => 'fieldpanel',
+
+            items => [
+                {   xtype       => 'passwordfield',
+                    name        => 'password',
+                    label       => l10n('New password'),
+                    placeholder => l10n('password'),
+                    required    => \1,
+                },
+                {   xtype       => 'passwordfield',
+                    name        => 'password1',
+                    label       => l10n('Confirm new password'),
+                    placeholder => l10n('password'),
+                    required    => \1,
+                },
+            ],
+
+            buttons => [
+                {   text    => l10n('Cancel'),
+                    ui      => 'decline',
+                    handler => 'close',
+                },
+                {   text    => l10n('Change'),
+                    ui      => 'confirm',
+                    handler => 'submit',
+                },
+            ]
+        } ]
+    };
+}
+
 1;
 __END__
 =pod
